/// <reference types="node" />
import { Command, flags as flagtype } from "@oclif/command";
import * as Utils from "../../lib/Utils";
declare type DecryptFileResult = Utils.ErrorOr<{
    decryptedContent: Buffer;
    destinationFileOutput?: string;
}>;
/**
 * File decrypt command. Takes a single or list of files and attempts to decrypt them. If a single file is provided without flags it will attempt to write the
 * decrypted file as a sibling of the encrypted file with it's extension removed. The -o option allows users to have the decrypted content to be written out
 * to either a different file location or `-o -` can be used to print the decrypted content to stdout. If multiple files are provided they will each be written
 * as siblings of their encrypted files with the file extension removed.
 */
export default class Decrypt extends Command {
    static description: string;
    static strict: boolean;
    static args: {
        name: string;
        description: string;
    }[];
    static flags: {
        help: import("@oclif/parser/lib/flags").IBooleanFlag<void>;
        keyfile: flagtype.IOptionFlag<string | undefined>;
        out: flagtype.IOptionFlag<string | undefined>;
        stdin: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        delete: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
    };
    static examples: string[];
    /**
     * Strip off the last extension from the provided file path. Would remove .iron from both file.json.iron and file.iron
     */
    removeLastFileExtension(filePath: string): string;
    /**
     * Read all the data from stdin into a single Buffer since we don't yet support streaming decryption. Returns a Promise which
     * resolves with all file data as bytes or rejects with any error encountered while reading the stream.
     */
    readAllContentFromStdin(): Promise<Buffer>;
    /**
     * Returns the bytes to decrypt conditionally based off whether the user is decrypting a file provided on the CLI or from content
     * read from stdin.
     */
    getBytesToDecrypt(decryptOptions: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<Buffer>;
    /**
     * If provided, verify the provided output file. If not provided, just resolves.
     */
    verifyOutfile(outPath?: string): Promise<void>;
    /**
     * Given an encrypted file path, read in the file content to bytes then extract the document ID from the doc
     * and use that to decrypt the document. Returns the decrypted document response from the SDK
     */
    getDocumentIDAndDecrypt(encryptedBytes: Buffer): Promise<import("@ironcorelabs/ironnode").DecryptedDocumentResponse>;
    /**
     * Returns a Promise of decrypting a file. Checks the file permissions of the provided input and output paths and attempts
     * to decrypt the file and write the result to the output file. If no output file is provided we take the same path as the
     * input file and strip off it's last extension.
     */
    getFileDecryptPromise(decryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<DecryptFileResult>;
    /**
     * Decrypt a collection of files and write them out to the same location but with their extension removed.
     */
    decryptMultipleFiles(decryptOp: Utils.ProcessMultipleFileOp): Promise<void>;
    /**
     * Decrypt a single file and write the decrypted bytes either to a sibling file, the location requested by the -o option or
     * write the bytes to stdout.
     */
    decryptSingleFile(decryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<void>;
    /**
     * Manual validation of complex flag and argument combinations to make sure the user is doing the right thing. Also limit the number of files that
     * we attempt to decrypt at any one time.
     */
    checkFlagsAndArguments(argv: string[], stdin: boolean, out?: string): undefined;
    run(): Promise<void>;
}
export {};
