/// <reference types="node" />
import { Command, flags as flagtype } from "@oclif/command";
import { DocumentAccessList, DocumentIDNameResponse } from "@ironcorelabs/ironnode";
import * as Utils from "../../lib/Utils";
declare type EncryptResult = Utils.ErrorOr<DocumentIDNameResponse>;
/**
 * File encrypt command. Takes a single file or multiple files and encrypts them to the caller and any optional users and
 * groups provided via flags. Multiple files and single files without the -o flag will write a file as a sibling of the plaintext
 * file with a `.iron` suffix. Providing the -o option allows users to write files to a specific file location or to use `-` to
 * write the encrypted bytes to stdout.
 */
export default class Encrypt extends Command {
    static description: string;
    static strict: boolean;
    static args: {
        name: string;
        description: string;
    }[];
    static flags: {
        help: import("@oclif/parser/lib/flags").IBooleanFlag<void>;
        keyfile: flagtype.IOptionFlag<string | undefined>;
        users: flagtype.IOptionFlag<string[] | undefined>;
        groups: flagtype.IOptionFlag<string[] | undefined>;
        out: flagtype.IOptionFlag<string | undefined>;
        stdin: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        delete: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
    };
    static examples: string[];
    /**
     * Get the readable stream that the content to encrypt is coming from. Either uses the process.stdin stream or verifies that the
     * provided sourcePath has the proper permissions and creates a readable stream from it.
     */
    getSourceStream(encryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<NodeJS.ReadableStream>;
    /**
     * Get the writable stream that the encrypted content will be piped out to. Either uses process.stdout stream or creates a writable
     * stream from either the outputFlag path or the sourcePath with a .iron extension.
     */
    getDestinationStream(encryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<NodeJS.WritableStream>;
    /**
     * Run both operations to get the source and destinations that we'll be reading/writing from for this operation.
     */
    getSourceAndDestinationStreams(encryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<(NodeJS.ReadableStream | NodeJS.WritableStream)[]>;
    /**
     * Returns a promise for the actual encrypt operation. Determines the source and destination streams and returns a Promise which runs
     * the encrypt operation. NOTE: The returned Promise from this method CANNOT fail as we let the caller decide how to handle failure. This
     * lets us prevent a complete failure when encrypting multiple files when one of them fails.
     */
    getFileEncryptPromise(fileAccessList: DocumentAccessList, encryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<EncryptResult>;
    /**
     * Encrypt a list of files to the provided list of users and groups. Runs a series of Promises in parallel and logs the results of the successes
     * and failures.
     */
    encryptMultipleFiles(fileAccessList: DocumentAccessList, encryptOp: Utils.ProcessMultipleFileOp): Promise<void>;
    /**
     * Encrypt a single file from it's path or from stdin and write the output to either
     * + A sibling of the source file with a .iron extension,
     * + The file path provided by the -o flag
     * + Stdout if the -o flag was set to '-'
     */
    encryptSingleFile(fileAccessList: DocumentAccessList, encryptOp: Utils.ProcessSingleFileOp | Utils.ProcessStdinOp): Promise<void>;
    /**
     * Various argument/outflag/stdinflag checks to verify that we support the combination of things they're trying to do.
     */
    checkFlagsAndArguments(argv: string[], outFlag?: string, stdinFlag?: boolean): undefined;
    run(): Promise<void>;
}
export {};
