"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const fs = require("fs");
const ironnode_1 = require("@ironcorelabs/ironnode");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
const Utils_1 = require("../../lib/Utils");
/**
 * File grant command. Lets user grant access to a file or list of files to a list of groups and/or users.
 */
class Grant extends command_1.Command {
    /**
     * Build a Promise that shares the provided file with the provide list of users and groups. Returns a fixed grant response
     * so that we can partially fail the grant operation.
     */
    getGrantPromise(filePath, accessList) {
        const sourceFile = Utils_1.normalizePathToFile(filePath);
        return Utils_1.checkSourceFilePermissions(sourceFile)
            .then(() => SDK_1.ironnode().document.getDocumentIDFromStream(fs.createReadStream(sourceFile)))
            .then((documentID) => {
            if (!documentID) {
                return Promise.reject(new Error(`Could not parse encrypted file.`));
            }
            return SDK_1.ironnode().document.grantAccess(documentID, accessList);
        })
            .then((shareResults) => ({
            source: sourceFile,
            shares: shareResults,
        }))
            .catch((e) => {
            return new Error(e.code === ironnode_1.ErrorCodes.DOCUMENT_HEADER_PARSE_FAILURE
                ? `Failed to revoke '${sourceFile}'. File doesn't appear to be an ironhide encrypted file.`
                : e.message);
        });
    }
    async run() {
        const { argv, flags } = this.parse(Grant);
        if (!flags.groups && !flags.users) {
            this.error(chalk_1.default.red("You must provide at least one user or group."));
        }
        const [groupsByName, groupsByID] = await GroupMaps.getGroupMaps();
        const requestedGroups = await GroupMaps.convertGroupNamesToIDs(flags.groups, groupsByName);
        const resultTable = Utils_1.createDisplayTable(["File", "Successful Grants", "Failed Grants"]);
        const accessList = Utils_1.convertUserAndGroupToAccessList(flags.users, requestedGroups);
        const grantOps = argv.map((file) => this.getGrantPromise(file, accessList));
        //Run each file grant operation and for operations that completly failed display an error but for
        //successful results, we return the list of successful grants and failures which get displayed within
        //a table.
        Promise.all(grantOps)
            .then((grantResultList) => {
            grantResultList.forEach((grantResult) => {
                if (Utils_1.isError(grantResult)) {
                    this.log(chalk_1.default.red(grantResult.message));
                }
                else {
                    resultTable.push(Utils_1.fileAccessResponseToTableRow(grantResult.source, grantResult.shares, groupsByID));
                }
            });
            if (resultTable.length) {
                this.log(resultTable.toString());
            }
        })
            .catch((e) => this.error(e.message));
    }
}
Grant.description = "Grant decryption access to a file or list of files to additional users or groups.";
Grant.strict = false;
Grant.args = [
    {
        name: "files",
        description: "Path of file or files to grant access to.",
        required: true,
    },
];
Grant.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
    users: command_1.flags.option({
        char: "u",
        description: "Grant access to the file(s) to a comma-separated list of user emails.",
        parse: (list) => list.split(","),
    }),
    groups: command_1.flags.option({
        char: "g",
        description: "Grant access to the file(s) to a comma-separated list of groups.",
        parse: (list) => list.split(","),
    }),
};
Grant.examples = [
    Utils_1.buildCommandSampleText("file:grant -u john@example.com -g myGroup path/to/file.iron", "Add decrypt access to the specified file to the provided user and group."),
    Utils_1.buildCommandSampleText("file:grant -u john@example.com,mike@example.com -g myGroup1,myGroup2 path/to/file.iron", "Add decrypt access to the specified file to multiple users and groups."),
    Utils_1.buildCommandSampleText("file:grant -g myGroup *.iron", "Add decrypt access to all of the '.iron' files in the current directory to 'myGroup'."),
];
exports.default = Grant;
