"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ironnode_1 = require("@ironcorelabs/ironnode");
const fs = require("fs");
const path_1 = require("path");
const chalk_1 = require("chalk");
const sharedFlags_1 = require("../../lib/sharedFlags");
const GroupMaps = require("../../lib/GroupMaps");
const Utils_1 = require("../../lib/Utils");
const SDK_1 = require("../../lib/SDK");
/**
 * File info command. Takes a single or multiple files as arguments and parses them for their IDs, then takes those IDs and looks up the meta information
 * from the SDk about the files. Displays results in a table.
 */
class Info extends command_1.Command {
    /**
     * Return a Promise to handle getting the meta information for a single file given it's relative path. Returns a result
     * structure that represents a success or error response.
     */
    getFileMetaPromise(filePath) {
        const sourceFile = Utils_1.normalizePathToFile(filePath);
        return Utils_1.checkSourceFilePermissions(sourceFile)
            .then(() => SDK_1.ironnode().document.getDocumentIDFromStream(fs.createReadStream(sourceFile)))
            .then((documentID) => {
            if (!documentID) {
                return Promise.reject(new Error("Could not parse encrypted file."));
            }
            return SDK_1.ironnode().document.getMetadata(documentID);
        })
            .then((documentMeta) => ({
            source: sourceFile,
            metadata: documentMeta,
        }))
            .catch((e) => {
            return new Error(e.code === ironnode_1.ErrorCodes.DOCUMENT_HEADER_PARSE_FAILURE
                ? `Failed to parse '${sourceFile}'. File doesn't appear to be an encrypted file.`
                : e.message);
        });
    }
    /**
     * Take the list of file meta results and groups by ID and create the display table, one row for each file we're getting information on.
     */
    buildResultTable(fileMeta, groupsByID) {
        const resultTable = Utils_1.createDisplayTable(["File", "Users with access", "Groups with access", "Created", "Updated"]);
        fileMeta.forEach((fileInfo) => {
            if (Utils_1.isError(fileInfo)) {
                this.log(chalk_1.default.red(fileInfo.message));
            }
            else {
                const { visibleTo } = fileInfo.metadata;
                resultTable.push([
                    path_1.basename(fileInfo.source),
                    visibleTo.users.map((user) => user.id).join("\n"),
                    visibleTo.groups.map((group) => `${groupsByID[group.id].groupName} (${group.id})`).join("\n"),
                    new Date(fileInfo.metadata.created).toLocaleDateString(),
                    new Date(fileInfo.metadata.updated).toLocaleDateString(),
                ]);
            }
        });
        return resultTable;
    }
    async run() {
        const { argv } = this.parse(Info);
        const metaOps = argv.map((file) => this.getFileMetaPromise(file));
        Promise.all(metaOps)
            .then((fileMeta) => {
            return GroupMaps.getGroupMaps()
                .then(([, groupsByID]) => {
                const resultTable = this.buildResultTable(fileMeta, groupsByID);
                if (resultTable.length) {
                    this.log(resultTable.toString());
                }
            })
                .catch((e) => {
                //TODO: what do we do here? In this case we've failed to get a map of the users group IDs so we can't display
                //the name properly.
                this.error(chalk_1.default.red(e.message));
            });
        })
            .catch((e) => this.error(e.message));
    }
}
Info.description = "Display information about an encrypted file or list of files.";
Info.strict = false;
Info.args = [
    {
        name: "files",
        description: "Path of file or files to display information for.",
        required: true,
    },
];
Info.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
Info.examples = [Utils_1.buildCommandSampleText("file:info path/to/file"), Utils_1.buildCommandSampleText("file:info *.iron")];
exports.default = Info;
