"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const fs = require("fs");
const ironnode_1 = require("@ironcorelabs/ironnode");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
const Utils_1 = require("../../lib/Utils");
/**
 * File revoke command. Lets user revoke access to a file or list of files from a list of groups and/or users.
 */
class Revoke extends command_1.Command {
    /**
     * Build a Promise that revokes access to the provided file with the provide list of users and groups. Returns a fixed revoke response
     * so that we can partially fail the revoke operation.
     */
    getRevokePromise(filePath, accessList) {
        const sourceFile = Utils_1.normalizePathToFile(filePath);
        return Utils_1.checkSourceFilePermissions(sourceFile)
            .then(() => SDK_1.ironnode().document.getDocumentIDFromStream(fs.createReadStream(sourceFile)))
            .then((documentID) => {
            if (!documentID) {
                return Promise.reject(new Error(`Could not parse encrypted file.`));
            }
            return SDK_1.ironnode().document.revokeAccess(documentID, accessList);
        })
            .then((shareResults) => ({
            source: sourceFile,
            revokes: shareResults,
        }))
            .catch((e) => {
            return new Error(e.code === ironnode_1.ErrorCodes.DOCUMENT_HEADER_PARSE_FAILURE
                ? `Failed to parse '${sourceFile}'. Document doesn't appear to be an encrypted file.`
                : e.message);
        });
    }
    async run() {
        const { argv, flags } = this.parse(Revoke);
        if (!flags.groups && !flags.users) {
            this.error(chalk_1.default.red("You must provide at least one user or group from which to revoke access."));
        }
        const resultTable = Utils_1.createDisplayTable(["File", "Successful Revokes", "Failed Revokes"]);
        const [groupsByName, groupsByID] = await GroupMaps.getGroupMaps();
        const requestedGroups = await GroupMaps.convertGroupNamesToIDs(flags.groups, groupsByName);
        const accessList = Utils_1.convertUserAndGroupToAccessList(flags.users, requestedGroups);
        const revokeOps = argv.map((file) => this.getRevokePromise(file, accessList));
        //Run each file revoke operation and for operations that completly failed display an error but for
        //successful results, we return the list of successful revokes and failures which get displayed within
        //a table.
        Promise.all(revokeOps)
            .then((revokeResultList) => {
            revokeResultList.forEach((revokeResult) => {
                if (Utils_1.isError(revokeResult)) {
                    this.log(chalk_1.default.red(revokeResult.message));
                }
                else {
                    resultTable.push(Utils_1.fileAccessResponseToTableRow(revokeResult.source, revokeResult.revokes, groupsByID));
                }
            });
            if (resultTable.length) {
                this.log(resultTable.toString());
            }
        })
            .catch((e) => this.error(e.message));
    }
}
Revoke.description = "Revoke cryptographic access to a file or list of files from other users or groups. (Note: revocation can also be accomplished by removing group members if you’re a group admin).";
Revoke.strict = false;
Revoke.args = [
    {
        name: "files",
        description: "Path of file or files to revoke access from.",
        required: true,
    },
];
Revoke.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
    users: command_1.flags.option({
        char: "u",
        description: "Revoke access to the file(s) from a comma-separated list of user emails.",
        parse: (list) => list.split(","),
    }),
    groups: command_1.flags.option({
        char: "g",
        description: "Revoke access to the file(s) from a comma-separated list of groups.",
        parse: (list) => list.split(","),
    }),
};
Revoke.examples = [
    Utils_1.buildCommandSampleText("file:revoke -u john@example.com -g myGroup path/to/file.iron", "Revoke access to the provided encrypted file from the provided user and group."),
    Utils_1.buildCommandSampleText("file:revoke -u john@example.com,mike@example.com -g myGroup1,myGroup2 path/to/file.iron", "Revoke access to the provided encrypted file from the provided users and groups."),
    Utils_1.buildCommandSampleText("file:revoke -g myGroup *.iron", "Revoke access to all of the '.iron' files from 'myGroup'."),
];
exports.default = Revoke;
