"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ironnode_1 = require("@ironcorelabs/ironnode");
const chalk_1 = require("chalk");
const Utils_1 = require("../../lib/Utils");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Group add admin command. Takes a comma-separated list of emails as admins as well as the group name and attempts to add
 * users to the group as admins.
 */
class AddAdmin extends command_1.Command {
    /**
     * Build up a table that displays the results from the group add admin operation showing both successes and failures.
     */
    buildResultTable(buildResults) {
        const table = Utils_1.createDisplayTable(["User", "Result"]);
        buildResults.succeeded.map((addedUser) => {
            table.push([addedUser, chalk_1.default.green("✔ Added as admin")]);
        });
        buildResults.failed.map((addedUser) => {
            table.push([addedUser.id, chalk_1.default.red(addedUser.error)]);
        });
        return table;
    }
    async run() {
        const { args, flags } = this.parse(AddAdmin);
        const groupID = await GroupMaps.getGroupIDFromName(args.group);
        SDK_1.ironnode()
            .group.addAdmins(groupID, flags.users)
            .then((addResult) => {
            this.log(`\n${this.buildResultTable(addResult).toString()}\n`);
        })
            .catch((e) => {
            if (e.code === ironnode_1.ErrorCodes.GROUP_ADD_ADMINS_NOT_ADMIN_FAILURE) {
                return this.error(chalk_1.default.red(`\nYou aren't an admin of the '${args.group}' group therefore you cannot manage its admins.\n`));
            }
            if (e.code === ironnode_1.ErrorCodes.GROUP_ADD_ADMINS_REQUEST_FAILURE) {
                return this.error(chalk_1.default.red(`\nUnable to complete request to add admins to the group.\n`));
            }
            this.error(chalk_1.default.red(e.message));
        });
    }
}
AddAdmin.aliases = ["group:addadmins", "group:addAdmin", "group:addAdmins"];
AddAdmin.description = "Add an admin to a group using their email address. Admins will be able to manage the group name, members, admins, and to delete the group.";
AddAdmin.args = [
    {
        name: "group",
        description: "Name of the group. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
];
AddAdmin.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
    users: sharedFlags_1.userList("Add admin permissions to the comma-separated list of user emails.")(),
};
AddAdmin.examples = [Utils_1.buildCommandSampleText("group:addadmin -u test@example.com,test2@example.com myGroup")];
exports.default = AddAdmin;
