"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ironnode_1 = require("@ironcorelabs/ironnode");
const chalk_1 = require("chalk");
const Utils_1 = require("../../lib/Utils");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Group add member command. Takes a comma-separated list of email addresses and attempts to add them as members to the group.
 */
class AddMember extends command_1.Command {
    /**
     * Build up a table that displays the results from the group add member operation showing both successes and failures.
     */
    buildResultTable(buildResults) {
        const table = Utils_1.createDisplayTable(["User", "Result"]);
        buildResults.succeeded.map((addedUser) => {
            table.push([addedUser, chalk_1.default.green("✔ Added as member")]);
        });
        buildResults.failed.map((addedUser) => {
            table.push([addedUser.id, chalk_1.default.red(addedUser.error)]);
        });
        return table;
    }
    async run() {
        const { args, flags } = this.parse(AddMember);
        const groupID = await GroupMaps.getGroupIDFromName(args.group);
        SDK_1.ironnode()
            .group.addMembers(groupID, flags.users)
            .then((addResult) => {
            this.log(`\n${this.buildResultTable(addResult).toString()}\n`);
        })
            .catch((e) => {
            if (e.code === ironnode_1.ErrorCodes.GROUP_ADD_MEMBER_NOT_ADMIN_FAILURE) {
                return this.error(chalk_1.default.red(`\nYou aren't an admin of the '${args.group}' group therefore you cannot manage its members.\n`));
            }
            if (e.code === ironnode_1.ErrorCodes.GROUP_ADD_MEMBERS_REQUEST_FAILURE) {
                return this.error(chalk_1.default.red(`\nUnable to complete request to add members to the group.\n`));
            }
            this.error(chalk_1.default.red(e.message));
        });
    }
}
AddMember.aliases = ["group:addmembers", "group:addMember", "group:addMembers"];
AddMember.description = "Add members to a group by email address. Members can decrypt all content that has been shared with the group.";
AddMember.args = [
    {
        name: "group",
        description: "Name of the group. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
];
AddMember.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
    users: sharedFlags_1.userList("Add member permissions to the comma-separated list of user emails.")(),
};
AddMember.examples = [Utils_1.buildCommandSampleText("group:addmember -u test@example.com,test2@example.com myGroup")];
exports.default = AddMember;
