"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ironnode_1 = require("@ironcorelabs/ironnode");
const chalk_1 = require("chalk");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const Utils_1 = require("../../lib/Utils");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Group create command. Creates a new group with the user provided ID. Auto adds the creator of the group as a member and an admin.
 */
class Create extends command_1.Command {
    /**
     * Check if the user is already a member/admin of a group with the provided name and fail if they are or
     * if we can't get the list of groups they're in.
     */
    async checkGroupExists(groupName) {
        let doesGroupExist = false;
        try {
            doesGroupExist = await GroupMaps.doesGroupNameAlreadyExist(groupName);
        }
        catch (e) {
            this.log(e);
            this.error(chalk_1.default.red("Unable to make group create request."));
        }
        if (doesGroupExist) {
            this.error(chalk_1.default.red(`You are already in a group with the name '${groupName}'.`));
        }
    }
    async run() {
        const { args } = this.parse(Create);
        const groupName = args.group;
        if (groupName.includes(",") || groupName.includes("^")) {
            return this.error("Group names cannot contain commas or carets.");
        }
        await this.checkGroupExists(args.group);
        SDK_1.ironnode()
            .group.create({ groupName: args.group })
            .then((group) => {
            const table = Utils_1.createDisplayTable(["Group Name", "Group ID", "Admin", "Member"]);
            table.push([group.groupName, group.groupID, chalk_1.default.green("✔"), chalk_1.default.green("✔")]);
            this.log(chalk_1.default.green("\nNew group successfully created."));
            this.log(chalk_1.default.green(`\n${table.toString()}\n`));
        })
            .catch((e) => {
            if (e.code === ironnode_1.ErrorCodes.GROUP_CREATE_REQUEST_FAILURE) {
                this.error(chalk_1.default.red("Unable to make group create request."));
            }
            this.error(chalk_1.default.red(e.message));
        });
    }
}
Create.description = `Create a new cryptographic group. Upon creation of the group you will become both an admin and a member of the group. This generates a public key for the group and uploads it to the IronCore service.`;
Create.args = [
    {
        name: "group",
        description: "Name for the group. Will be used when referencing this group from all other commands.",
        required: true,
    },
];
Create.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
Create.examples = [Utils_1.buildCommandSampleText("group:create myGroup", "Create a new group with the name 'myGroup'")];
exports.default = Create;
