"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const SDK_1 = require("../../lib/SDK");
const sharedFlags_1 = require("../../lib/sharedFlags");
const GroupMaps = require("../../lib/GroupMaps");
class Delete extends command_1.Command {
    /**
     * Deleting a group is currently a risky operation as it nukes the group from the DB. Display a warning to the user on delete telling them the
     * downsides of their decision and make them re-enter the group name to verify the deletion of the group.
     */
    async verifyGroupName(enteredGroupName, groupID) {
        let group;
        try {
            // tslint:disable-next-line
            group = (await SDK_1.ironnode().group.get(groupID));
            this.log(chalk_1.default.yellowBright(`\nWarning! Deleting a group will cause all documents encrypted to only that group to no longer be decryptable! The group you are trying to delete has ${group.groupAdmins.length} admin(s) and ${group.groupMembers.length} member(s).\n`));
        }
        catch (e) {
            this.error(chalk_1.default.red("Was not able to retrieve information about the provided group."));
        }
        const groupConfirm = await cli_ux_1.default.prompt(chalk_1.default.magenta("Please enter the group to delete again to confirm its deletion"));
        if (groupConfirm !== enteredGroupName) {
            throw new Error(`Group confirmation failed. Original group provided was '${enteredGroupName}' but confirmation value was '${groupConfirm}'.`);
        }
    }
    async run() {
        const { args } = this.parse(Delete);
        const groupID = await GroupMaps.getGroupIDFromName(args.group);
        const [, groupsByID] = await GroupMaps.getGroupMaps();
        if (!groupsByID[groupID].isAdmin) {
            return this.error(chalk_1.default.red(`You aren't currently an admin of '${args.group}' so you may not delete it.`));
        }
        try {
            await this.verifyGroupName(args.group, groupID);
        }
        catch (e) {
            return this.error(chalk_1.default.red(e.message));
        }
        SDK_1.ironnode()
            .group.deleteGroup(groupID)
            .then(() => this.log(chalk_1.default.green("Group successfully deleted!")))
            .catch((e) => {
            this.log(e.message);
            this.error(chalk_1.default.red("Group delete request failed."));
        });
    }
}
Delete.description = "Delete a group given its name or ID. Once deleted all files encrypted to only the group will no longer be able to be decrypted.";
Delete.args = [
    {
        name: "group",
        description: "Name of the group. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
];
Delete.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
exports.default = Delete;
