"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const Table = require("cli-table3");
const Utils_1 = require("../../lib/Utils");
const SDK_1 = require("../../lib/SDK");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Group info command. Retrieves a group given its name or ID and displays information about the group including the
 * members and admins if the requesting user is an admin.
 */
class Info extends command_1.Command {
    /**
     * Typeguard to determine which type of group detail response we have.
     */
    canSeeGroupDetails(group) {
        return group.isAdmin || group.isMember;
    }
    /**
     * Build up the display table for group details. Displays the list of members and admins if the current
     * user is an admin of the group.
     */
    buildGroupDetailTable(group) {
        const table = new Table();
        const check = chalk_1.default.green("✔");
        const nope = chalk_1.default.red("✖");
        table.push({ [chalk_1.default.blue("Group")]: group.groupName }, { [chalk_1.default.blue("ID")]: group.groupID }, { [chalk_1.default.blue("Admin")]: group.isAdmin ? check : nope }, { [chalk_1.default.blue("Member")]: group.isMember ? check : nope }, { [chalk_1.default.blue("Created")]: new Date(group.created).toLocaleDateString() }, { [chalk_1.default.blue("Updated")]: new Date(group.updated).toLocaleDateString() });
        if (this.canSeeGroupDetails(group)) {
            table.push({ [chalk_1.default.blue("Admins")]: group.groupAdmins.join("\n") }, { [chalk_1.default.blue("Members")]: group.groupMembers.join("\n") });
        }
        return table;
    }
    async run() {
        const { args } = this.parse(Info);
        const groupID = await GroupMaps.getGroupIDFromName(args.group);
        try {
            const group = await SDK_1.ironnode().group.get(groupID);
            this.log(`\n${this.buildGroupDetailTable(group).toString()}\n`);
        }
        catch (e) {
            this.error(chalk_1.default.red(`Group '${args.group}' couldn't be retrieved.`));
        }
    }
}
Info.description = "Get detailed information about a group.";
Info.args = [
    {
        name: "group",
        description: "Name of the group to retrieve. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
];
Info.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
Info.examples = [
    Utils_1.buildCommandSampleText("group:info myGroup"),
    Utils_1.buildCommandSampleText("group:info id^b94850b8855486ca504ecbc521801153", "Display information about a group given its ID"),
];
exports.default = Info;
