"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const ironnode_1 = require("@ironcorelabs/ironnode");
const SDK_1 = require("../../lib/SDK");
const sharedFlags_1 = require("../../lib/sharedFlags");
const GroupMaps = require("../../lib/GroupMaps");
const Utils_1 = require("../../lib/Utils");
/**
 * Group remove member command. Takes a comma-separated list of emails as well as the group name and attempts to remove users as members from the group.
 */
class RemoveMember extends command_1.Command {
    /**
     * Build up a table that displays the results from the group remove member operation showing both successes and failures.
     */
    buildResultTable(buildResults) {
        const table = Utils_1.createDisplayTable(["User", "Result"]);
        buildResults.succeeded.map((removedUser) => {
            table.push([removedUser, chalk_1.default.green("✔ Removed as member")]);
        });
        buildResults.failed.map((removedUser) => {
            table.push([removedUser.id, chalk_1.default.red(removedUser.error)]);
        });
        return table;
    }
    async run() {
        const { args, flags } = this.parse(RemoveMember);
        const groupID = await GroupMaps.getGroupIDFromName(args.group);
        SDK_1.ironnode()
            .group.removeMembers(groupID, flags.users)
            .then((removeResult) => {
            this.log(`\n${this.buildResultTable(removeResult).toString()}\n`);
        })
            .catch((e) => {
            if (e.code === ironnode_1.ErrorCodes.GROUP_REMOVE_MEMBERS_REQUEST_FAILURE) {
                return this.error(chalk_1.default.red(`Unable to complete request to remove members from the group.`));
            }
            this.error(chalk_1.default.red(e.message));
        });
    }
}
RemoveMember.aliases = ["group:removemembers", "group:removeMember", "group:removeMembers"];
RemoveMember.description = "Remove members from a group given their email address.";
RemoveMember.args = [
    {
        name: "group",
        description: "Name of the group. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
];
RemoveMember.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
    users: sharedFlags_1.userList("Remove member permissions from the comma-separated list of user emails.")(),
};
RemoveMember.examples = [Utils_1.buildCommandSampleText("group:removemember -u test@example.com,test2@example.com myGroup")];
exports.default = RemoveMember;
