"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const SDK_1 = require("../../lib/SDK");
const Utils_1 = require("../../lib/Utils");
const GroupMaps = require("../../lib/GroupMaps");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Change the name of a group given its existing name or ID and a new name to update it to.
 */
class Rename extends command_1.Command {
    /**
     * Check if the user is already a member/admin of a group with the provided name that they're changing to.
     */
    async checkGroupExists(groupName) {
        let doesGroupExist = false;
        try {
            doesGroupExist = await GroupMaps.doesGroupNameAlreadyExist(groupName);
        }
        catch (e) {
            this.log(e);
            this.error(chalk_1.default.red("Unable to make group rename request."));
        }
        if (doesGroupExist) {
            this.error(chalk_1.default.red(`You are already in a group with the name '${groupName}'. Please pick a different name.`));
        }
    }
    async run() {
        const { args } = this.parse(Rename);
        if (args.newGroupName.includes(",") || args.newGroupName.includes("^")) {
            return this.error("Group names cannot contain commas or carets.");
        }
        await this.checkGroupExists(args.newGroupName);
        const groupID = await GroupMaps.getGroupIDFromName(args.currentGroupName);
        SDK_1.ironnode()
            .group.update(groupID, { groupName: args.newGroupName })
            .then(() => this.log(chalk_1.default.green("Group name successfully updated!")))
            .catch((e) => {
            this.log(e);
            this.error(chalk_1.default.red("Group could not be updated."));
        });
    }
}
Rename.description = "Change the name of a group. Won't change any of the group admins, members, or files encrypted to the group.";
Rename.args = [
    {
        name: "currentGroupName",
        description: "Current name of the group. Can alternately refer to a group by ID. Indicate IDs by prefixing with 'id^' e.g. 'id^groupID'.",
        required: true,
    },
    {
        name: "newGroupName",
        description: "New name of the group.",
        required: true,
    },
];
Rename.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
Rename.examples = [Utils_1.buildCommandSampleText("group:rename myGroup newGroup")];
exports.default = Rename;
