"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const SDK_1 = require("../../lib/SDK");
const Utils_1 = require("../../lib/Utils");
const sharedFlags_1 = require("../../lib/sharedFlags");
/**
 * Delete one to many devices given their ID. Verifies that the user isn't trying to use this command to delete the device they're on and makes requests
 * in parallel to delete devices for each ID provided.
 */
class DeviceDelete extends command_1.Command {
    /**
     * Return a Promise operation to delete the provided device ID. Maps any failures into successes so we can display information about each
     * result back to the user.
     */
    getDeviceDeletePromise(deviceID) {
        const numericalDeviceID = parseInt(deviceID);
        if (!numericalDeviceID) {
            return Promise.resolve(new Error(`Expected a numerical device ID but got '${deviceID}' instead.`));
        }
        return SDK_1.ironnode()
            .user.deleteDevice(numericalDeviceID)
            .catch((e) => e);
    }
    /**
     * Check if the user is attempting to delete the device keys that they're currently using. If so display an error.
     */
    async checkDeletionOfCurrentDevice(deviceIDs) {
        let devices;
        try {
            devices = await SDK_1.ironnode().user.listDevices();
        }
        catch (e) {
            throw new Error("Failed to make request to get current device keys.");
        }
        const currentDevice = devices.result.find((device) => device.isCurrentDevice);
        if (currentDevice && deviceIDs.includes(`${currentDevice.id}`)) {
            throw new Error("Attempting to delete keys for the current device. Use the 'logout' command instead.");
        }
    }
    async run() {
        const { argv } = this.parse(DeviceDelete);
        try {
            await this.checkDeletionOfCurrentDevice(argv);
        }
        catch (e) {
            return this.error(chalk_1.default.red(e.message));
        }
        const deviceDeletes = argv.map((deviceID) => this.getDeviceDeletePromise(deviceID));
        return Promise.all(deviceDeletes).then((resultList) => {
            let successfulCount = 0;
            let failureCount = 0;
            resultList.forEach((deviceDeleteResult) => {
                if (Utils_1.isError(deviceDeleteResult)) {
                    this.log(chalk_1.default.red(deviceDeleteResult.message));
                    failureCount++;
                }
                else {
                    successfulCount++;
                }
            });
            if (successfulCount > 0) {
                this.log(chalk_1.default.green(`\n${successfulCount} device key(s) successfully deleted.`));
            }
            if (failureCount > 0) {
                this.log(chalk_1.default.red(`\n${failureCount} device key(s) failed to be deleted.`));
            }
        });
    }
}
DeviceDelete.aliases = ["user:deviceDelete"];
DeviceDelete.description = "Deauthorize a device from being able to decrypt your data. Use this to deauthorize devices other than the device you are currently on. To deauthorize the current device, use 'ironhide logout'.";
DeviceDelete.args = [
    {
        name: "device ID",
        description: "ID of the device keys to revoke. Multiple IDs can be provided.",
        required: true,
    },
];
DeviceDelete.strict = false;
DeviceDelete.flags = {
    help: command_1.flags.help({ char: "h" }),
    keyfile: sharedFlags_1.keyFile(),
};
DeviceDelete.examples = [Utils_1.buildCommandSampleText(`user:devicedelete 293 5559 312`)];
exports.default = DeviceDelete;
