"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@oclif/errors");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const SDK_1 = require("./SDK");
const Utils_1 = require("./Utils");
exports.GROUP_ID_PREFIX = "id^";
let hasRequestedGroups = false;
let groupsByIndexCache = {
    groupsByName: {},
    groupsByID: {},
};
/**
 * Check if the provided group ID is actually a fixed ID value.
 */
function isGroupID(providedGroupValue) {
    return providedGroupValue.startsWith(exports.GROUP_ID_PREFIX);
}
/**
 * Typeguard to check whether the provided group-by-name value is an array of groups or a single group.
 */
function isMultipleGroups(groupList) {
    return Array.isArray(groupList);
}
/**
 * Convert a list of groups from the SDK response into a map from the provided index (name or ID) to the group details.
 */
function createGroupMapByIndex(groupResult) {
    const groupsByIndex = {
        groupsByName: {},
        groupsByID: {},
    };
    return groupResult.reduce((maps, group) => {
        maps.groupsByID[group.groupID] = group;
        //If the group doesn't have a name (will only happen if the group was created outside of the CLI) we just omit it from the by-name map. The user
        //will have to provide the group by ID in that case.
        if (!group.groupName) {
            return maps;
        }
        const groupName = group.groupName;
        const groupAtIndex = maps.groupsByName[groupName];
        if (Array.isArray(groupAtIndex)) {
            groupAtIndex.push(group);
        }
        else if (groupAtIndex) {
            maps.groupsByName[groupName] = [groupAtIndex, group];
        }
        else {
            maps.groupsByName[groupName] = group;
        }
        return maps;
    }, groupsByIndex);
}
/**
 * Get a list of the users groups and create maps from the results to groups keyed by ID and groups keyed by name in the local cache
 * so we can store them for later.
 */
function populateUsersGroups() {
    if (hasRequestedGroups) {
        return Promise.resolve([groupsByIndexCache.groupsByName, groupsByIndexCache.groupsByID]);
    }
    return SDK_1.ironnode()
        .group.list()
        .then((groupResult) => {
        groupsByIndexCache = createGroupMapByIndex(groupResult.result);
        hasRequestedGroups = true;
        return [groupsByIndexCache.groupsByName, groupsByIndexCache.groupsByID];
    });
}
/**
 * Get a value from the user in a prompt and make sure they entered a valid number for a choice of which group
 * to select.
 */
async function getGroupChoice(size) {
    let choice = "1";
    try {
        const questionRange = chalk_1.default.gray(`(1 - ${size})`);
        choice = await cli_ux_1.default.prompt(`${chalk_1.default.magenta("Enter a choice")} ${questionRange}`);
    }
    catch (_) {
        //User exited out, so bail
        process.exit(0);
    }
    const numericalChoice = parseInt(choice);
    if (numericalChoice && numericalChoice > 0 && numericalChoice <= size) {
        return Promise.resolve(numericalChoice - 1);
    }
    console.log(chalk_1.default.red("Invalid option, please try again."));
    return getGroupChoice(size);
}
/**
 * Given a list of groups with duplicate names, get the underlying ID of the group the user wants to use for this
 * operation.
 */
async function getUsersGroupChoice(groupName, groups) {
    return new Promise((resolve) => {
        const check = chalk_1.default.green("✔");
        const nope = chalk_1.default.red("✖");
        console.log(chalk_1.default.yellow(`\nMultiple groups found with the provided name '${groupName}', which one do you want to use?\n`));
        const groupDetailsTable = Utils_1.createDisplayTable(["Option", "ID", "Admin", "Member", "Created", "Updated"]);
        groups.forEach((group, index) => {
            groupDetailsTable.push([
                index + 1,
                group.groupID,
                group.isAdmin ? check : nope,
                group.isMember ? check : nope,
                new Date(group.created).toLocaleDateString(),
                new Date(group.updated).toLocaleDateString(),
            ]);
        });
        console.log(groupDetailsTable.toString());
        getGroupChoice(groups.length)
            .then((groupIndexChoice) => resolve(groups[groupIndexChoice].groupID))
            .catch(() => process.exit(-1));
    });
}
/**
 * Clear group list cache. Used only within unit tests.
 */
function clearCache() {
    hasRequestedGroups = false;
    groupsByIndexCache = {
        groupsByName: {},
        groupsByID: {},
    };
}
exports.clearCache = clearCache;
/**
 * Get a map from group ID to group information for all the groups the user is a part of.
 */
async function getGroupMaps() {
    let groupsByID;
    let groupsByName;
    try {
        [groupsByName, groupsByID] = await populateUsersGroups();
    }
    catch (e) {
        console.log(e);
        throw new errors_1.CLIError("Unable to make request to lookup group information.");
    }
    return [groupsByName, groupsByID];
}
exports.getGroupMaps = getGroupMaps;
/**
 * Take a list of group names and map them to a list of group IDs. If any name provided can't be mapped to a name
 * it will come back unmodified.
 */
async function convertGroupNamesToIDs(groupNames, groupsByName) {
    if (!groupNames || !groupNames.length) {
        return Promise.resolve([]);
    }
    //Iterate through each group and build up a Promise chain of all the group names to IDs. The promises will all be auto resolved unless
    //we find a group name which has a duplicate. In that case we'll ask the user for their choice before proceeding.
    const resolvedNames = groupNames.reduce(async (promiseChain, providedName) => {
        let resolvedNameToID;
        if (isGroupID(providedName)) {
            resolvedNameToID = () => Promise.resolve(providedName.substring(exports.GROUP_ID_PREFIX.length));
        }
        else {
            const groupAtIndex = groupsByName[providedName];
            resolvedNameToID = () => {
                if (groupAtIndex) {
                    if (isMultipleGroups(groupAtIndex)) {
                        return getUsersGroupChoice(providedName, groupAtIndex);
                    }
                    return Promise.resolve(groupAtIndex.groupID);
                }
                return Promise.resolve(providedName);
            };
        }
        return promiseChain.then((chainResults) => {
            return resolvedNameToID().then((currentResult) => [...chainResults, currentResult]);
        });
    }, Promise.resolve([]));
    return resolvedNames;
}
exports.convertGroupNamesToIDs = convertGroupNamesToIDs;
/**
 * Given a group name, lookup the groups that the user is a part of and attempt to get the ID of
 * the group. Will return null if user isn't a member or admin of any group with the provided name.
 */
async function getGroupIDFromName(groupName) {
    if (isGroupID(groupName)) {
        return Promise.resolve(groupName.substring(exports.GROUP_ID_PREFIX.length));
    }
    let groupsByName;
    try {
        [groupsByName] = await populateUsersGroups();
    }
    catch (_) {
        throw new errors_1.CLIError("Unable to make request for provided group.");
    }
    if (!groupsByName[groupName]) {
        throw new errors_1.CLIError(`Group '${groupName}' doesn't exist or couldn't be retrieved.`);
    }
    const groupAtIndex = groupsByName[groupName];
    if (isMultipleGroups(groupAtIndex)) {
        return getUsersGroupChoice(groupName, groupAtIndex);
    }
    return Promise.resolve(groupAtIndex.groupID);
}
exports.getGroupIDFromName = getGroupIDFromName;
/**
 * Given a group name, see if the user is already part of a group with the same name.
 */
function doesGroupNameAlreadyExist(groupName) {
    return populateUsersGroups().then(([groupsByName]) => {
        return groupsByName[groupName] !== undefined;
    });
}
exports.doesGroupNameAlreadyExist = doesGroupNameAlreadyExist;
