"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path_1 = require("path");
const Table = require("cli-table3");
const chalk_1 = require("chalk");
/**
 * Type guard to determine if the file options we have represents a multiple file operation.
 */
function isMultipleFileOperation(fileProcessOp) {
    return Array.isArray(fileProcessOp.file);
}
exports.isMultipleFileOperation = isMultipleFileOperation;
/**
 * Type guard to determine if the file options we have represent an operation from stdin.
 */
function isStdInFileOperation(fileProcessOp) {
    return !fileProcessOp.file;
}
exports.isStdInFileOperation = isStdInFileOperation;
/**
 * Type guard to determine of ErrorOr result is an Error instance.
 */
function isError(result) {
    return result instanceof Error;
}
exports.isError = isError;
/**
 * Returns whether or not the file provided both exists and is readable.
 */
function isFileReadable(path) {
    try {
        fs.accessSync(path, fs.constants.R_OK);
        return true;
    }
    catch (_) {
        return false;
    }
}
exports.isFileReadable = isFileReadable;
/**
 * Check to see if the provided device key config file is parsable and formatted correctly.
 */
function validateExistingKeys(configPath) {
    try {
        const config = JSON.parse(fs.readFileSync(configPath, "utf8"));
        return config && config.accountID && config.segmentID && config.deviceKeys && config.signingKeys;
    }
    catch (e) {
        return false;
    }
}
exports.validateExistingKeys = validateExistingKeys;
/**
 * Attempt to normalize and create a fully qualified path given either an existing fully qualified path
 * or a partial path. If the provided path starts with / it assumes it's already fully qualified, otherwise
 * it normalizes the path from the cwd.
 */
function normalizePathToFile(file) {
    if (file.startsWith(path_1.sep)) {
        return file;
    }
    return path_1.normalize(path_1.join(process.cwd(), file));
}
exports.normalizePathToFile = normalizePathToFile;
/**
 * Attempt to delete the provided partial file path. Will throw an exception if the file isn't writable and cannot be deleted.
 */
function deleteFile(filePath) {
    const sourceFile = normalizePathToFile(filePath);
    fs.accessSync(sourceFile, fs.constants.W_OK);
    fs.unlinkSync(sourceFile);
}
exports.deleteFile = deleteFile;
/**
 * Validate that the provided file to encrypt is readable and is an actual file.
 */
function checkSourceFilePermissions(fileToEncrypt) {
    //Make sure we can read from the source file
    if (!isFileReadable(fileToEncrypt)) {
        return Promise.reject(new Error(`Provided path '${fileToEncrypt}' doesn't exist or is not readable.`));
    }
    //Make sure the provided input file is not a directory
    const sourceFileStats = fs.lstatSync(fileToEncrypt);
    if (!sourceFileStats.isFile()) {
        return Promise.reject(new Error(`Provided path '${fileToEncrypt}' does not appear to be a file.`));
    }
    return Promise.resolve();
}
exports.checkSourceFilePermissions = checkSourceFilePermissions;
/**
 * Check that the file path to write to doesn't already exist and the directory it will live in is writable.
 */
function checkDestinationFilePermissions(destinationFile) {
    //Make sure the destination file doesn't already exist
    if (fs.existsSync(destinationFile)) {
        return Promise.reject(new Error(`Output path '${destinationFile}' already exists.`));
    }
    //Make sure we can write to the output location
    try {
        fs.accessSync(path_1.dirname(destinationFile), fs.constants.W_OK);
        return Promise.resolve();
    }
    catch (_) {
        return Promise.reject(new Error(`Output path '${path_1.dirname(destinationFile)}' is not writable.`));
    }
}
exports.checkDestinationFilePermissions = checkDestinationFilePermissions;
/**
 * Given an optional list of users and groups convert them into an object that the SDK takes for user and group shares.
 */
function convertUserAndGroupToAccessList(providedUserIDs, providedGroupIDs) {
    let userShares = [];
    let groupShares = [];
    if (providedUserIDs && providedUserIDs.length) {
        userShares = providedUserIDs.map((userEmail) => ({ id: userEmail }));
    }
    if (providedGroupIDs && providedGroupIDs.length) {
        groupShares = providedGroupIDs.map((groupID) => ({ id: groupID }));
    }
    return { users: userShares, groups: groupShares };
}
exports.convertUserAndGroupToAccessList = convertUserAndGroupToAccessList;
/**
 * Creates a new CLI table from the provided columns. Centralized method to return the proper type as well as change
 * the table headers from their default of red (bad!) to blue (nice!).
 */
function createDisplayTable(columns) {
    return new Table({
        head: columns.map((columnLabel) => chalk_1.default.blue(columnLabel)),
    });
}
exports.createDisplayTable = createDisplayTable;
/**
 * Convert a grant/revoke response object into a CLI table for which displays the list of successes and failures
 */
function fileAccessResponseToTableRow(fileSource, accessResult, groupsByID) {
    const { succeeded, failed } = accessResult;
    const successfulIDToDisplay = ({ id, type }) => {
        return type === "user" ? id : groupsByID[id].groupName;
    };
    const failedIDToDisplay = ({ id, type, error }) => {
        //If we're displaying a user error or a group error but we don't have the ID which likely means that they tried
        //to share with a group that doesn't exist.
        if (type === "user" || !groupsByID[id]) {
            return `${id} (${error})`;
        }
        //Display the group name instead of the ID. Also replace the group ID in the error message as it might contain it.
        const groupName = groupsByID[id].groupName;
        return `${groupName} (${error.replace(id, groupName)})`;
    };
    return [path_1.basename(fileSource), chalk_1.default.green(succeeded.map(successfulIDToDisplay).join("\n")), chalk_1.default.red(failed.map(failedIDToDisplay).join("\n"))];
}
exports.fileAccessResponseToTableRow = fileAccessResponseToTableRow;
/**
 * Build up example command text which support for providing an optional description of the example. All examples are prefixed with a `$ ironhide` to avoid
 * duplication of that for each command.
 */
function buildCommandSampleText(example, description, showPrefix = true) {
    const descText = description ? `\n${chalk_1.default.gray(description)} \n ` : "";
    const commandPrefix = showPrefix ? chalk_1.default.green("ironhide") : "";
    return `${descText} ${chalk_1.default.magenta("$")} ${commandPrefix} ${example}`;
}
exports.buildCommandSampleText = buildCommandSampleText;
